package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.junmp.jyzb.api.bean.query.ApplicationReq;
import com.junmp.jyzb.api.exception.enums.CabinetExceptionEnum;
import com.junmp.jyzb.entity.Application;
import com.junmp.jyzb.service.ApplicationService;
import com.junmp.jyzb.mapper.ApplicationMapper;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import com.junmp.v2.db.api.factory.PageFactory;
import com.junmp.v2.db.api.factory.PageResultFactory;
import com.junmp.v2.db.api.page.PageResult;
import org.springframework.stereotype.Service;

/**
* @author lxh专属坐骑
* @description 针对表【sys_application】的数据库操作Service实现
* @createDate 2023-09-11 14:59:13
*/
@Service
public class ApplicationServiceImpl extends ServiceImpl<ApplicationMapper, Application>
    implements ApplicationService{

    //获取应用的分页信息
    @Override
    public PageResult<Application> GetAllUpdateCfg(ApplicationReq req) {
        LambdaQueryWrapper<Application> wrapper = createWrapper(req);
        Page<Application> page = page(PageFactory.getDefaultPage(req.getPageNo(), req.getPageSize()), wrapper);
        return PageResultFactory.createPageResult(page);
    }

    //新增应用信息
    @Override
    public String AddOrUpdateCfg(ApplicationReq req) {
        Application application = new Application();
        BeanPlusUtil.copyProperties(req,application);
        //判断id是否传递，如果传递则默认他是修改，根据数据库对比id，判断是否id传递错误；如果不传递默认新增
        if (req.getId()==null || req.getId().trim().isEmpty()){

            save(application);
            return application.getId();
        }else {
            Application app = getById(req);
            if (ObjectUtil.isNull(app)){
                throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
            }
            updateById(application);
            return application.getId();
        }

    }

    private LambdaQueryWrapper<Application> createWrapper(ApplicationReq req) {
        LambdaQueryWrapper<Application> wrapper = new LambdaQueryWrapper<>();
        if (ObjectUtil.isEmpty(req)) {
            return wrapper;
        }
        wrapper.like(ObjectUtil.isNotEmpty(req.getName()),Application::getName,req.getName());
        wrapper.eq(ObjectUtil.isNotEmpty(req.getType()),Application::getType,req.getType());
        wrapper.orderByDesc(Application::getCreateTime);
        return wrapper;

    }
}




