package com.junmp.jyzb.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.junmp.jyzb.api.bean.dto.OrderDetailDto;
import com.junmp.jyzb.api.bean.dto.OrderDto;
import com.junmp.jyzb.api.bean.dto.OrderMainDto;
import com.junmp.jyzb.api.bean.query.OrderMainReq;
import com.junmp.jyzb.api.bean.query.OrderUploadReq;
import com.junmp.jyzb.api.bean.req.DetailListReq;
import com.junmp.jyzb.api.bean.req.UpdateOrderReq;
import com.junmp.jyzb.api.exception.enums.CabinetExceptionEnum;
import com.junmp.jyzb.api.exception.enums.OrderExceptionEnum;
import com.junmp.jyzb.entity.*;
import com.junmp.jyzb.mapper.OrderMainMapper;
import com.junmp.jyzb.service.OrderDetailService;
import com.junmp.jyzb.service.OrderNumService;
import com.junmp.jyzb.service.OrderMainService;
import com.junmp.jyzb.service.PubOrgService;
import com.junmp.jyzb.utils.DateTimeUtil;
import com.junmp.v2.common.exception.base.ServiceException;
import com.junmp.v2.common.util.BeanPlusUtil;
import com.junmp.v2.db.api.factory.PageFactory;
import com.junmp.v2.db.api.factory.PageResultFactory;
import com.junmp.v2.db.api.page.PageResult;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.core.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;


@Service
public class OrderMainServiceImpl extends ServiceImpl<OrderMainMapper, OrderMain> implements OrderMainService {

    @Resource
    private OrderDetailService orderDetailService;
    @Resource
    private RabbitTemplate rabbitTemplate;
    @Resource
    private RabbitAdmin rabbitAdmin;

    @Resource
    private OrderNumService orderNumService;

    @Resource
    private PubOrgService pubOrgService;



    //新增任务单
    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> AddOrder(UpdateOrderReq req) {
        //存放主单据和子单据，便于返回
        List<String> list=new ArrayList<>();
        OrderMain order = new OrderMain();
        BeanPlusUtil.copyProperties(req, order);
        //设置单据单号（订单号）
        OrderNum orderNum = setOrderCode(req);
        String codeValue=String.format("%04d",orderNum.getNum());
        order.setOrderCode(orderNum.getBussinessType()+"-"+orderNum.getOutInType()+"-"+orderNum.getYear()+"-"+codeValue);
        //设置总价格和应出入库数量
        //设置id
        String id=UUID.randomUUID().toString();
        order.setId(id);
        Integer sumNum=0;
        BigDecimal priceTotal=new BigDecimal(0);
        List<OrderDetail> detailList = new ArrayList<>();
        //通过遍历批量保存详细信息
        for (DetailListReq listReq:req.getDetailList()) {
            OrderDetail detail=new OrderDetail();
            BeanPlusUtil.copyProperties(listReq, detail);
            detail.setOrderId(order.getId());
            //设置单据类型
            detail.setType(order.getOrderType());
            //将数量和价格进行累加
            sumNum += detail.getPlanNum();
            priceTotal = priceTotal.add(BigDecimal.valueOf(detail.getPlanNum()).multiply(detail.getPrice())) ;
            detail.setCreateTime(DateTimeUtil.getCurrentDateTime());
            detailList.add(detail);
        }
        orderDetailService.saveBatch(detailList);
       //设置总价格和总数量
        order.setPrice(priceTotal);
        order.setInventoryQuantity(sumNum);
        order.setCreateTime(DateTimeUtil.getCurrentDateTime());
        //保存
        this.save(order);
        list.add(order.getId());
        for (OrderDetail orderDetail:detailList) {
            list.add(Long.toString(orderDetail.getId()));
        }
        return list;
    }


    //设置单据单号（并且对order_num表进行新增或者更新）
    public OrderNum setOrderCode(UpdateOrderReq req){
        //设置采购单号，需要先判断该组织机构的采购单号是否存在，如果存在则将数量进行增加，如果不存在则新增一条对应的数据
        OrderNum orderNum = new OrderNum();
        //获取年份，获取业务出入状态，获取组织机构id，获取业务类型
        orderNum.setYear(LocalDateTime.now().getYear());
        //判断是出库还是入库，出库为发物单位，入库为收物单位
        if (req.getOrderType().equals("in")){
            orderNum.setOrgId(Long.valueOf(req.getEndOrgId()));
        }else {
            orderNum.setOrgId(Long.valueOf(req.getStartOrgId()));
        }
        //业务类型
        orderNum.setBussinessType(req.getBussinessType());
        //出入业务状态
        orderNum.setOutInType(req.getOrderType());

        OrderNum one = orderNumService.getOne(new LambdaQueryWrapper<OrderNum>()
                .eq(OrderNum::getOrgId, orderNum.getOrgId())
                .eq(OrderNum::getBussinessType, orderNum.getBussinessType())
                .eq(OrderNum::getYear, orderNum.getYear())
                .eq(OrderNum::getOutInType, orderNum.getOutInType()));
        if (ObjectUtil.isNull(one)){
            orderNum.setNum(1);
            orderNumService.save(orderNum);
            return orderNum;
        }else {
            one.setNum(one.getNum()+1);
            orderNumService.updateById(one);
            return one;
        }

    }


    //新增默认审核通过的任务单
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String AddFinishOrder(UpdateOrderReq req) {
        //添加到数据库中
        String orderId = AddOrder(req).get(0);
        //将完成的任务单直接推送到消息队列rabbitmq中（需要判断是入库单还是出库单，入库单传发物单位id，出库单传收物单位id）
        //方法1：一个交换机，一个队列。通过中间对象，object存储对象，type表标识
//        if(req.getOrderType().equals("in")){
//            rabbitTemplate.convertAndSend("OrderExchange","OrderRouting",new MessageWrapper(req,req.getStartOrgId()));
//        }else {
//            rabbitTemplate.convertAndSend("OrderExchange","OrderRouting",new MessageWrapper(req,req.getEndOrgId()));
//        }
        //方法2：一个交换机，多个队列。动态创建队列
        String exchangeName="orderExchange";
        //需要判断是入库单还是出库单，入库单传收物单位id，出库单传发物单位id
        if(req.getOrderType().equals("in")){
            Queue queue=new Queue(req.getEndOrgId(),true,false,false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(req.getEndOrgId()).noargs());
            rabbitTemplate.convertAndSend(exchangeName, req.getEndOrgId(), req);

        }else {
            Queue queue = new Queue(req.getStartOrgId(), true, false, false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(req.getStartOrgId()).noargs());
            rabbitTemplate.convertAndSend(exchangeName, req.getStartOrgId(), req);
        }
        return orderId;
    }


    //更新任务单（进入工作流中之后不能进行更改）
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean updateOrder(UpdateOrderReq req) {
        OrderMain order = orderMainExist(req.getId());
        if (!order.getExamineState().equals("none"))
        {
            throw new ServiceException(OrderExceptionEnum.ORDER_CAN_NOT_UPDATE);
        }
        List<OrderDetail> list= orderDetailService.QueryList(req.getId());
        //idSet中存放的是存在数据库中的任务单中的orderDetail中的id（原始的工作单中的数据id）
        Set<String> idSet =new HashSet<>();
        for (OrderDetail orderDetail:list){
            idSet.add(Long.toString(orderDetail.getId()));
        }
        //将该任务单的所有id转换为String,并添加到updateList中，用来判断删除或者修改新增的是否存在
        //判断删除列表是否为空，如果不为空则进行批量删除
        if (ObjectUtil.isNotNull(req.getDeleteList())&& ObjectUtil.isNotEmpty(req.getDeleteList())) {
            //判断传递的list Id是否存在，如果不存在则抛出异常
            for (String s:req.getDeleteList()){
                if (idSet.contains(s)){
                    //将要删除的装备价格和数据进行查询出来，并别对任务单的总数量和总价格进行更改
                    OrderDetail one = orderDetailService.getOne(new LambdaQueryWrapper<OrderDetail>()
                            .eq(OrderDetail::getOrderId, s));
                    order.setPrice(order.getPrice().subtract(one.getPrice().multiply(BigDecimal.valueOf(one.getPlanNum()))));
                    order.setInventoryQuantity(order.getInventoryQuantity() - one.getPlanNum());
                    orderDetailService.removeBatchByIds(req.getDeleteList());
                }else {
                    throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
                }
            }
        }
        //对detaillist进行处理
        List<OrderDetail> addList = new ArrayList<>();//新增的
        List<OrderDetail> updateList = new ArrayList<>();//更新
        for (DetailListReq detailListReq :req.getDetailList()) {
            //判断是否存在，存在则进行修改
            if (idSet.contains(detailListReq.getId())){
                OrderDetail one = orderDetailService.getOne(new LambdaQueryWrapper<OrderDetail>()
                        .eq(OrderDetail::getId, detailListReq.getId()));
                BeanPlusUtil.copyProperties(detailListReq,one);
                one.setType(req.getOrderType());
                //判断数量和价格是否有变动，如果有变动，则将order的总价和总数量进行更新操作，否则不变
                if (one.getPlanNum().equals(detailListReq.getPlanNum()) && one.getPrice().compareTo(detailListReq.getPrice())!=0){
                    order.setPrice(order.getPrice().subtract(one.getPrice().multiply(BigDecimal.valueOf(one.getPlanNum())))
                            .add(detailListReq.getPrice().multiply(BigDecimal.valueOf(detailListReq.getPlanNum()))));
                    order.setInventoryQuantity(order.getInventoryQuantity() - one.getPlanNum()+ detailListReq.getPlanNum());
                }
                one.setUpdateTime(DateTimeUtil.getCurrentDateTime());
                updateList.add(one);
            }else {
                //否则判断id是否为空或者错误，如果为空则添加，不为空则抛出异常
                if (ObjectUtil.isNull(detailListReq.getId()) || detailListReq.getId().equals("") || ObjectUtil.isEmpty(detailListReq.getId())){
                    OrderDetail detail=new OrderDetail();
                    BeanPlusUtil.copyProperties(detailListReq, detail);
                    detail.setOrderId(req.getId());
                    detail.setType(req.getOrderType());
                    //将数量和价格进行相加处理
                    order.setPrice(order.getPrice().add(detailListReq.getPrice().multiply(BigDecimal.valueOf(detailListReq.getPlanNum()))));
                    order.setInventoryQuantity(order.getInventoryQuantity() + detailListReq.getPlanNum());
                    detail.setUpdateTime(DateTimeUtil.getCurrentDateTime());
                    addList.add(detail);
                }else {
                    throw new ServiceException(CabinetExceptionEnum.PARAMETER_ERROR);
                }

            }
        }
        orderDetailService.updateBatchById(updateList);
        orderDetailService.saveBatch(addList);
        BeanPlusUtil.copyProperties(req,order);
        return this.updateById(order);
    }
    public boolean hasCommonElements(List<?> list1, List<?> list2) {
        for (Object item1 : list1) {
            if (list2.contains(item1)) {
                return true; // 找到相同值
            }
        }
        return false; // 没有相同值
    }

    //查询任务列表(根据组织机构id)
    @Override
    public PageResult<OrderMainDto> getOrderPage(OrderMainReq req) {
        //判断组织机构是否存在
        pubOrgService.PubOrgExist(req.getOrgId());
        //判断是出库单还是入库单，选择的查询条件也不相同
        Page<OrderMain> page=new Page<>();
        LambdaQueryWrapper<OrderMain> wrapper = createWrapper(req);
        if (req.getOrderType() != null && req.getOrderType().equals("in")){//入库单
            LambdaQueryWrapper<OrderMain> eq = wrapper.eq(OrderMain::getOrderType, req.getOrderType())
                    .eq(OrderMain::getEndOrgId, req.getOrgId())
//                    .eq()
//                    .eq()
//                    .eq()
                    .orderByDesc(OrderMain::getCreateTime);
            page=this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),eq);
        }
        if (req.getOrderType() != null && req.getOrderType().equals("out")) {//出库单
            LambdaQueryWrapper<OrderMain> eq = wrapper.eq(OrderMain::getOrderType, req.getOrderType()).eq(OrderMain::getStartOrgId, req.getOrgId());
            page=this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),eq);
        }
        List<OrderMainDto> orderMainDtoList=new ArrayList<>();
        //遍历page,(如果出入库的组织机构为同一个的时候需要进行去重处理，不然会存在两条重复的数据)
        for (OrderMain orderMain : page.getRecords()){
            OrderMainDto orderMainDto = new OrderMainDto();
            BeanPlusUtil.copyProperties(orderMain,orderMainDto);
            if (!orderMainDtoList.contains(orderMainDto)){
                orderMainDtoList.add(orderMainDto);
            }
        }
        Page<OrderMainDto> page1 = PageFactory.getDefaultPage(req.getPageNo(), req.getPageSize());
        page1.setTotal(orderMainDtoList.size());
        page1.setRecords(orderMainDtoList);
        return PageResultFactory.createPageResult(page1);
    }

//    //查询任务列表(根据组织机构id)
//    @Override
//    public PageResult<OrderMainDto> getOrderPage(OrderMainReq req) {
//        //判断组织机构是否存在
//        pubOrgService.PubOrgExist(req.getOrgId());
//        //判断是出库单还是入库单，选择的查询条件也不相同
//        Page<OrderMain> page=new Page<>();
//        LambdaQueryWrapper<OrderMain> wrapper = createWrapper(req);
//        if (req.getOrderType() != null && req.getOrderType().equals("in")){//入库单
//            LambdaQueryWrapper<OrderMain> eq = wrapper.eq(OrderMain::getOrderType, req.getOrderType())
//                    .eq(OrderMain::getEndOrgId, req.getOrgId());
//            page=this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),eq);
//        } else if (req.getOrderType() != null && req.getOrderType().equals("out")) {//出库单
//            LambdaQueryWrapper<OrderMain> eq = wrapper.eq(OrderMain::getOrderType, req.getOrderType()).eq(OrderMain::getStartOrgId, req.getOrgId());
//            page=this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),eq);
//        }else {//查询全部
//            Page<OrderMain> page1=new Page<>();
//            Page<OrderMain> page2=new Page<>();
//            LambdaQueryWrapper<OrderMain> eq1 = wrapper.eq(OrderMain::getEndOrgId, req.getOrgId());
//            page1=this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),eq1);
//            LambdaQueryWrapper<OrderMain> eq2 = wrapper.eq(OrderMain::getStartOrgId, req.getOrgId());
//            page2=this.page(PageFactory.getDefaultPage(req.getPageNo(),req.getPageSize()),eq2);
//            List<OrderMain> list = new ArrayList<>();
//            list.addAll(page1.getRecords());
//            list.addAll(page2.getRecords());
//            page.setRecords(list);
//        }
//        List<OrderMainDto> orderMainDtoList=new ArrayList<>();
//        //遍历page,(如果出入库的组织机构为同一个的时候需要进行去重处理，不然会存在两条重复的数据)
//        for (OrderMain orderMain : page.getRecords()){
//            OrderMainDto orderMainDto = new OrderMainDto();
//            BeanPlusUtil.copyProperties(orderMain,orderMainDto);
//            if (!orderMainDtoList.contains(orderMainDto)){
//                orderMainDtoList.add(orderMainDto);
//            }
//        }
//        Page<OrderMainDto> page1 = PageFactory.getDefaultPage(req.getPageNo(), req.getPageSize());
//        page1.setTotal(orderMainDtoList.size());
//        page1.setRecords(orderMainDtoList);
//        return PageResultFactory.createPageResult(page1);
//    }

    //根据任务单id查看业务明细
    @Override
    public OrderDto GetDetailById(OrderMainReq req) {
        //判断任务单是否存在
        OrderMain orderMain = orderMainExist(req.getId());
        OrderDto orderDto = new OrderDto();
        BeanPlusUtil.copyProperties(orderMain,orderDto);
        //将详细信息存入列表属性中
        //通过任务单id查询对应的详细信息
        List<OrderDetail> list = orderDetailService.list(new LambdaQueryWrapper<OrderDetail>()
                .eq(OrderDetail::getOrderId, req.getId()));
        List<OrderDetailDto> orderDetailDtoList=new ArrayList<>();
        for (OrderDetail orderDetail:list) {
            OrderDetailDto orderDetailDto = new OrderDetailDto();
            BeanPlusUtil.copyProperties(orderDetail,orderDetailDto);
            orderDetailDtoList.add(orderDetailDto);
        }
        orderDto.setDetailList(orderDetailDtoList);
        return orderDto;
    }

    //单据状态上报
    @Override
    public boolean PushState(OrderUploadReq req) {
        OrderMain orderMain = orderMainExist(req.getId());
        //获取单据的出入库，获取发物单位id或者收物单位id
        String orderType = orderMain.getOrderType();
        String exchangeName="OrderStateExchange";
        //推送到rabbitmq中去
        if (orderType.equals("in")){
            Long endOrgId = orderMain.getEndOrgId();
            Queue queue=new Queue(Long.toString(endOrgId),true,false,false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(Long.toString(endOrgId)).noargs());
            rabbitTemplate.convertAndSend(exchangeName, Long.toString(endOrgId), req);

        }else {
            Long startOrgId = orderMain.getStartOrgId();
            Queue queue = new Queue(Long.toString(startOrgId), true, false, false);
            Exchange exchange = new DirectExchange(exchangeName, true, false);
            rabbitAdmin.declareQueue(queue);
            rabbitAdmin.declareExchange(exchange);
            rabbitAdmin.declareBinding(BindingBuilder.bind(queue).to(exchange).with(Long.toString(startOrgId)).noargs());
            rabbitTemplate.convertAndSend(exchangeName, Long.toString(startOrgId), req);
        }
        return true;
    }

    //记账
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean Accounting(OrderUploadReq req) {
        //判断传递的订单主id是否存在
        OrderMain orderMain = orderMainExist(req.getId());
        //记账子单据不能传空
        if (req.getChildrenOrder().size()==0){
            throw new ServiceException(OrderExceptionEnum.ORDERDETAIL_ISNOT_NULL);
        }
        List<OrderUploadReq.OrderUploadDetailReq> childrenOrder = req.getChildrenOrder();
        //并且记录到数据表order_log中
        List<OrderLog> orderLogList=new ArrayList<>();
        //将实际数量进行添加
        Integer actualQuantity=0;
        //记账直接将单子结束
        orderMain.setOrderState("finished");
        //判断传递的子单据id在数据库中是否存在，如果不存在则抛出子单据不存在异常
        List<OrderDetail> list = orderDetailService.list(new LambdaQueryWrapper<OrderDetail>()
                .in(OrderDetail::getId, childrenOrder));
        Set<Long> existingIds = list.stream()
                .map(OrderDetail::getId)
                .collect(Collectors.toSet());
        List<OrderUploadReq.OrderUploadDetailReq> missingList = new ArrayList<>();
        for (OrderUploadReq.OrderUploadDetailReq uploadDetailReq : childrenOrder) {
            if (!existingIds.contains(uploadDetailReq.getDetailId())) {
                missingList.add(uploadDetailReq);
            }
            //遍历的时候记录每个子单据的实际数量,得到主单据的实际数量
            actualQuantity+=uploadDetailReq.getActualNum();
            //判断子单据实际数量和应入库数量是否相等,如果不相等记录下来并存于log表中
            if (!uploadDetailReq.getActualNum().equals(uploadDetailReq.getPlanNum())){
                OrderLog orderLog = new OrderLog();
                orderLog.setOrderId(req.getId());
                orderLog.setHistoryMsg("将"+uploadDetailReq.getOrderType()+"-"+
                        uploadDetailReq.getSizeName()+"的物资数量修改为"+uploadDetailReq.getActualNum());
                orderLogList.add(orderLog);
            }
        }
        if (missingList.size()!=0){
            throw new ServiceException(OrderExceptionEnum.ORDERDETAIL_NOT_EXIST);
        }
        //创建一个list用于存放修改的数据以便于进行批量操作

        //批量更新子单据数量
//        orderDetailService.updateBatchById();
        orderMain.setActualQuantity(actualQuantity);
        //更新主单据
        return updateById(orderMain);
    }


    //判断任务单是否存在
    @Override
    public OrderMain orderMainExist(String id) {
        OrderMain order = this.getById(id);
        if (ObjectUtil.isNull(order)) {
            throw new ServiceException(OrderExceptionEnum.ORDER_NOT_EXIST);
        }
        return order;
    }

    //设置pageNo和pageSize的通用方法
    public Map<String,Long> getPage(Long no,Long size){
        Map<String,Long> map=new HashMap<>();
        long pageSize;
        long pageNo;
        if (ObjectUtil.isNull(no)){
            pageNo = 1L;
        }else {
            pageNo= no;
        }
        if ( ObjectUtil.isNull(size)){
            pageSize = 20L;
        }else {
            pageSize= size;
        }
        map.put("pageNo",pageNo);
        map.put("pageSize",pageSize);
        return map;
    }



    private LambdaQueryWrapper<OrderMain> createWrapper(OrderMainReq req) {
        LambdaQueryWrapper<OrderMain> wrapper = new LambdaQueryWrapper<>();
        if (ObjectUtil.isEmpty(req)) {
            return wrapper;
        }
        //单据状态 ready待入/出库，finished已入/出库，working入/出库中
        wrapper.eq(ObjectUtil.isNotEmpty(req.getOrderState()), OrderMain::getOrderState, req.getOrderState());
        //审核状态:none,未接入审核流，working审批中，finished已完成
        wrapper.eq(ObjectUtil.isNotEmpty(req.getExamineState()), OrderMain::getExamineState, req.getExamineState());
        //查询出指定时间内的单据
//        wrapper.ge(ObjectUtil.isNotEmpty(req.getStartTime()),OrderMain::getCreateTime,)
        //时间降序
        wrapper.orderByDesc(OrderMain::getCreateTime);
        return wrapper;

    }

}
